#
# This file is part of pysimpledb.
#
# pysimpledb is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pysimpledb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with pysimpledb.  If not, see <http://www.gnu.org/licenses/>.
#
#      Author Daniel Mikusa <dan@mikusa.com>
#   Copyright Daniel Mikusa 2008
#

class AbstractRowMapper(object):
    def map(self, colNames, row, rownum):
        raise NotImplementedError("You cannot use this class directly")

class DictionaryRowMapper(AbstractRowMapper):

    DEFAULT_EXEC_TYPE = 'queryForList'
    
    def map(self, colNames, row, rownum):
        data = {}
        for col,val in zip(colNames, row):
            data[col] = val
        return data

class ListRowMapper(AbstractRowMapper):
    """Maps the values to a simple list"""

    DEFAULT_EXEC_TYPE = 'queryForList'

    def map(self, colNames, row, rownum):
        return list(row)

class ObjectRowMapper(AbstractRowMapper):
    """Maps the values to an Object"""

    DEFAULT_EXEC_TYPE = 'queryForList'

    def __init__(self, type):
        self.type = type

    def map(self, colNames, row, rownum):
        obj = self.type()
        for col,val in zip(colNames, row):
            setattr(obj, col, val)
        return obj

class SingleTypeRowMapper(AbstractRowMapper):
    """Map to a single value using a conversion function"""

    DEFAULT_EXEC_TYPE = 'queryForObject'

    def __init__(self, convert, col = 0):
        self.convert = convert
        self.col = col

    def map(self, colNames, row, rownum):
        if type(self.col) == int:
            return self.convert(row[self.col])
        elif type(self.col) == str:
            colNum = colNames.index(self.col)
            return self.convert(row[colNum])
        else:
            raise ValueError("Column Not Found!")
